/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase base para las clases trabajadoras. Implementa las
propiedades necesarias para armar correctamente la estructura
del modelo UML.
- ENGLISH -
Base class for worker classes. Implements the properties
needed to properly build UML structure.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.

/**
Se encarga de crear todos los objetos y paquetes necesarios para luego poder procesar las relaciones
entre ellos.
**/
CLASS com.nomadesoft.ABLaUML.ABLXMLWorker IMPLEMENTS IXMLWorker:
  DEFINE PRIVATE VARIABLE cinNextOrden AS INTEGER     NO-UNDO.
  
  DEFINE PROTECTED TEMP-TABLE cttObjectsIDs NO-UNDO
    FIELD inOrden AS INTEGER
    FIELD inID AS INTEGER
    INDEX ixOrden IS PRIMARY IS UNIQUE inOrden.

  DEFINE PROTECTED VARIABLE cobUtils AS CLASS IUtils NO-UNDO.

  DEFINE PUBLIC PROPERTY clgIncremental AS LOGICAL INITIAL TRUE NO-UNDO
    GET.
    SET.

  /* parser que invoc el procesamiento */
  DEFINE PROTECTED PROPERTY cobParser AS CLASS IXMLParser NO-UNDO
    GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinCurrentPackageID AS INTEGER NO-UNDO
    GET.
    SET.

  /* ruta al mdulo de clasificadores */
  DEFINE PUBLIC PROPERTY cchClasifPath AS CHARACTER INITIAL 'comm/' NO-UNDO
    GET.
    PROTECTED SET.
  DEFINE PUBLIC PROPERTY cchClasifMatch AS CHARACTER NO-UNDO
    GET():
      RETURN '*' + cchClasifPath + '*'.
    END GET.


  /* ruta al mdulo de librera */
  DEFINE PUBLIC PROPERTY cchLibPath AS CHARACTER INITIAL 'lib/' NO-UNDO
    GET.
    PROTECTED SET.
  DEFINE PUBLIC PROPERTY cchLibMatch AS CHARACTER NO-UNDO
    GET():
        RETURN '*' + cchLibPath + '*'.
    END GET.

  /* nombres de los paquetes, se buscan los nombres nslib,clasif,project,dm,comps y menu y se utiliza el elemento que le sigue para nombrar los paquetes
  en el repositorio */
  DEFINE PUBLIC PROPERTY cchPackageNamesMap AS CHARACTER INITIAL 'lib,Lib,common,Common,project,Project,dm,Data View,menu,Menu View,comps,Components View' NO-UNDO
    GET.
    SET.

  METHOD PROTECTED CHARACTER getPackName(INPUT ipchName AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    minIndex = LOOKUP(ipchName,cchPackageNamesMap).
    IF minIndex GT 0 THEN
      RETURN ENTRY(minIndex + 1,cchPackageNamesMap).
    RETURN ipchName.
  END METHOD.

  /* id del objeto que corresponde al marcador xml que se est procesando actualmente, es decir que corresponde al objeto padre del marcador actual */
  DEFINE PROTECTED PROPERTY cinCurrentObjectID AS INTEGER NO-UNDO
    GET.
    SET(INPUT ipinOID AS INTEGER):
      IF ipinOID EQ ? THEN DO:
        IF cinNextOrden <= 0 THEN DO:
          cobParser:DoLog(STRING(this-object),"Finalizacin de cinCurrentObjectID incorrecta","WARNING").
          RETURN.
        END.
        FIND cttObjectsIDs WHERE inOrden EQ cinNextOrden.
        cobParser:DoLog(STRING(this-object),"Finalizacin de cinCurrentObjectID:" + STRING(inOrden) + ' - ' + STRING(inID),"DEBUG").
        DELETE cttObjectsIDs.
        cinNExtOrden = cinNextOrden - 1.
        IF cinNextOrden GT 0 THEN DO:
          FIND cttObjectsIDs WHERE inOrden EQ cinNextOrden.
          cinCurrentObjectId = cttObjectsIDs.inID.
        END.
        ELSE
          cinCurrentObjectId = 0.
      END.
      ELSE DO:
        CREATE cttObjectsIDs.
        ASSIGN
          cinNextOrden = cinNextOrden + 1
          cttObjectsIDs.inOrden = cinNextOrden
          cttObjectsIDs.inID = ipinOID.
        cinCurrentObjectId = ipinOID.
        cobParser:DoLog(STRING(this-object),"Creacin de cinCurrentObjectID:" + STRING(inOrden) + ' - ' + STRING(inID) ,"DEBUG").
      END.
      cobParser:DoLog(STRING(this-object),"cinCurrentObjectID:" + STRING(cinCurrentObjectID) + ' - ' + STRING(cinNextOrden) ,"DEBUG").
    END SET.
  
    /* identificadores de los modelos que conforman el repositorio */
  DEFINE PROTECTED PROPERTY cinNSLibPackageId AS INTEGER NO-UNDO
    GET:
      IF cinNSLibPackageId EQ 0 THEN DO:
        cinNSLibPackageId = cobUtils:findPackage(getPackName('lib'),0).
        IF cinNSLibPackageId EQ 0 THEN
          cinNSLibPackageId = cobUtils:createModelPackage(getPackName('lib'),0,?).
      END.
      RETURN cinNSLibPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinProjectPackageId AS INTEGER NO-UNDO
    GET:
      IF cinProjectPackageId EQ 0 THEN DO:
        cinProjectPackageId = cobUtils:findPackage(getPackName('project'),0).
        IF cinProjectPackageId EQ 0 THEN
          cinProjectPackageId = cobUtils:createModelPackage(getPackName('project'),0,?).
      END.
      RETURN cinProjectPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinClasifPackageId AS INTEGER NO-UNDO
    GET:
      IF cinClasifPackageId EQ 0 THEN DO:
        cinClasifPackageId = cobUtils:findPackage(getPackName('common'),0).
        IF cinClasifPackageId EQ 0 THEN
          cinClasifPackageId = cobUtils:createModelPackage(getPackName('common'),0,?).
      END.
      RETURN cinClasifPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinDataModelPackageId AS INTEGER NO-UNDO
    GET:
      IF cinDataModelPackageId EQ 0 THEN DO:
        cinDataModelPackageId = cobUtils:findPackage(getPackName('dm'),cinProjectPackageId).
        IF cinDataModelPackageId EQ 0 THEN
          cinDataModelPackageId = cobUtils:createModelPackage(getPackName('dm'),cinProjectPackageId,3).
      END.
      RETURN cinDataModelPackageId.
    END GET.
    PRIVATE SET.

  /* id de las carpetas de componentes para cada modelo */
  DEFINE PROTECTED PROPERTY cinNSLibComponentsPackageId AS INTEGER NO-UNDO
    GET:
      IF cinNSLibComponentsPackageId EQ 0 THEN DO:
        cinNSLibComponentsPackageId = cobUtils:findPackage(getPackName('comps'),cinNSLibPackageId).
        IF cinNSLibComponentsPackageId EQ 0 THEN
          cinNSLibComponentsPackageId = cobUtils:createModelPackage(getPackName('comps'),cinNSLibPackageId,4).
      END.
      RETURN cinNSLibComponentsPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinProjectComponentsPackageId AS INTEGER NO-UNDO
    GET:
      IF cinProjectComponentsPackageId EQ 0 THEN DO:
        cinProjectComponentsPackageId = cobUtils:findPackage(getPackName('comps'),cinProjectPackageId).
        IF cinProjectComponentsPackageId EQ 0 THEN
          cinProjectComponentsPackageId = cobUtils:createModelPackage(getPackName('comps'),cinProjectPackageId,4).
      END.
      RETURN cinProjectComponentsPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinClasifComponentsPackageId AS INTEGER NO-UNDO
    GET:
      IF cinClasifComponentsPackageId EQ 0 THEN DO:
        cinClasifComponentsPackageId = cobUtils:findPackage(getPackName('comps'),cinClasifPackageId).
        IF cinClasifComponentsPackageId EQ 0 THEN
          cinClasifComponentsPackageId = cobUtils:createModelPackage(getPackName('comps'),cinClasifPackageId,4).
      END.
      RETURN cinClasifComponentsPackageId.
    END GET.
    PRIVATE SET.

  /* modelo de datos (tablas) */
  DEFINE PROTECTED PROPERTY cinDataModelComponentsPackageId AS INTEGER NO-UNDO
    GET:
      IF cinDataModelComponentsPackageId EQ 0 THEN DO:
        cinDataModelComponentsPackageId = cobUtils:findPackage(getPackName('comps'),cinDataModelPackageId).
        IF cinDataModelComponentsPackageId EQ 0 THEN
          cinDataModelComponentsPackageId = cobUtils:createModelPackage(getPackName('comps'),cinDataModelPackageId,3).
      END.
      RETURN cinDataModelComponentsPackageId.
    END GET.
    PRIVATE SET.


  CONSTRUCTOR ABLXMLWorker(INPUT ipobUtils AS CLASS IUtils):
    IF NOT VALID-OBJECT(ipobUtils) THEN
      UNDO, THROW NEW AppError('Se requiere un objeto que implemente la interfaz IRepositoryUtils',0101).
    cobUtils = ipobUtils.
  END CONSTRUCTOR.

  METHOD PROTECTED CHARACTER SplitDBTable(INPUT ipchCalificado AS CHARACTER, OUTPUT opchDB AS CHARACTER):
    IF NUM-ENTRIES(ipchCalificado,'.') < 2 THEN DO:
      opchDB = ?.
      RETURN ipchCalificado.
    END.
    opchDB = ENTRY(1,ipchCalificado,'.').
    RETURN ENTRY(2,ipchCalificado,'.').
  END METHOD.

  /* implementacin de IXMLWorker */
  METHOD PUBLIC VOID WorkStart(INPUT ipobParser AS CLASS IXMLParser):
    IF NOT VALID-OBJECT(ipobParser) THEN
      UNDO, THROW NEW AppError('IXMLParser no vlido',0201).
    cobParser = ipobParser.
    cobParser:doLog(STRING(THIS-OBJECT),'Comienzo del procesamiento','DEBUG').
  END METHOD.
  METHOD PUBLIC VOID WorkEnd(INPUT ipobParser AS CLASS IXMLParser):
    IF NOT VALID-OBJECT(ipobParser) THEN
      UNDO, THROW NEW AppError('IXMLParser no vlido',0202).
    cobParser = ipobParser.
    cobParser:doLog(STRING(THIS-OBJECT),'Fin del procesamiento','DEBUG').
  END METHOD.
  METHOD PUBLIC VOID ElementStart(INPUT ipobParser AS CLASS IXMLParser):
    IF NOT VALID-OBJECT(ipobParser) THEN
      UNDO, THROW NEW AppError('IXMLParser no vlido',0203).
    cobParser = ipobParser.
    cobParser:doLog(STRING(THIS-OBJECT),SUBSTITUTE('Comienzo elemento &1',QUOTER(cobParser:CurrentElement())),'DEBUG').
  END METHOD.
  METHOD PUBLIC VOID ElementEnd(INPUT ipobParser AS CLASS IXMLParser):
    IF NOT VALID-OBJECT(ipobParser) THEN
      UNDO, THROW NEW AppError('IXMLParser no vlido',0204).
    cobParser = ipobParser.
    cobParser:doLog(STRING(THIS-OBJECT),SUBSTITUTE('Fin elemento &1',QUOTER(cobParser:CurrentElement())),'DEBUG').
  END METHOD.
END CLASS.
