/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Limpia el modelo de datos UML, quitando todas las tablas que no
se utilicen en ningn diagrama de unidad funcional. Accede
directamente al repositorio UML.
- ENGLISH -
Cleans the UML data model, removing all tables not referenced in
any functional unit diagram. Requires direct access to Repository
database.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING com.nomadesoft.ABLaUML.*.
USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.lib.*.
USING Progress.Lang.AppError.

CLASS com.nomadesoft.ABLaUML.DictCleaner:
  DEFINE PRIVATE VARIABLE cobUtils AS IUtils    NO-UNDO.

  DEFINE PUBLIC PROPERTY cobStereotypeMapper AS IMapper NO-UNDO
    GET.
    SET(INPUT ipobMapper AS IMapper):
        cobStereotypeMapper = ipobMapper.
        CAST(cobUtils,EAUtils):cobStereotypeMapper = ipobMapper.
    END SET.

  METHOD PROTECTED CHARACTER DoMap(INPUT ipchStMap AS CHARACTER):
    IF VALID-OBJECT(cobStereotypeMapper) THEN
      ipchStMap = cobStereotypeMapper:MapIt(ipchStMap,FALSE).
    RETURN ipchStMap.
  END METHOD.

  CONSTRUCTOR PUBLIC DictCleaner(INPUT ipobLogger AS CLASS ILogger):
    cobUtils = NEW EAUtils(TRUE).
  END CONSTRUCTOR.

  /** destructor **/
  DESTRUCTOR DictCleaner():
    DELETE OBJECT cobUtils.
  END DESTRUCTOR.

  /* busca todos los objetos (tablas) del modelo de datos que no estn relacionados a otros objetos y los elimina */
  METHOD PUBLIC VOID CleanIt(INPUT iplgWipeDB AS LOGICAL,INPUT ipchDMName AS CHARACTER):
    define buffer mbfObject for Repository.t_Object.
    DEFINE BUFFER mbfChilds FOR Repository.T_Object.
    DEFINE BUFFER mbfDiagrams FOR Repository.T_Diagram.
    DEFINE VARIABLE mchStereotype AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchChildsSt AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minParent AS INTEGER INITIAL ? NO-UNDO.

    mchStereotype = DoMap('nsTable').
    FOR EACH mbfObject WHERE mbfObject.Object_Type EQ 'Class' AND mbfObject.Stereotype EQ mchStereotype:
      IF cobUtils:DataLinkCount(mbfObject.OBJECT_ID) EQ 0 THEN
        DELETE mbfObject.
    END.
    
    IF NOT iplgWipeDB THEN
        RETURN.
    /* buscar todas las bases de datos que no tengan tablas y eliminarlas del modelo */
    ASSIGN
        mchChildsSt = mchStereotype
        mchStereotype = DoMap('nsDatabase')
        minParent = cobUtils:findPackage(ipchDMName,?) WHEN ipchDMName NE ?.
    .
    FOR EACH mbfObject WHERE mbfObject.OBJECT_Type EQ 'Package' AND (mbfObject.Stereotype EQ mchStereotype OR (minParent NE ? AND mbfObject.Package_ID EQ minParent)):
        /* determinar cantidad de tablas que contiene */
        IF CAN-FIND(FIRST mbfChilds WHERE mbfChilds.OBJECT_type EQ 'Class' AND mbfChilds.Stereotype EQ mchChildsSt
            AND mbfChilds.Package_ID EQ INTEGER(mbfObject.pdata1)) THEN
            NEXT.
        /* eliminar cualquier hijo restante (que no son tablas) */
        FOR EACH mbfChilds WHERE mbfChilds.Package_ID EQ INTEGER(mbfObject.pdata1):
            DELETE mbfChilds.
        END.
        /* eliminar el diagrama */
        cobUtils:DeleteDiagram(INTEGER(mbfObject.pdata1),?).
        /* eliminar el paquete correspondiente a la base de datos */
        cobUtils:DeletePackage(INTEGER(mbfObject.pdata1),TRUE).
    END.
  END METHOD.
END CLASS.
