/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase para generar el XML con la informacin de nombres alternativos
de componentes (Alias), en base al contenido de la base de datos
xref y mapsissrc.
- ENGLISH -
Class for building components Alias XML, base on xref and mapsissrc
databases.
*****************/

ROUTINE-LEVEL ON ERROR UNDO, THROW.
USING com.nomadesoft.lib.XMLwriter.


CLASS com.nomadesoft.xml.Aliaswriter INHERITS XMLwriter:

  DEFINE PUBLIC PROPERTY cchModulePaths AS CHARACTER NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY cchCurrentPath AS CHARACTER NO-UNDO
    GET.
    PROTECTED SET.

  METHOD PROTECTED VOID SetModulePath(INPUT ipchNombre AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    cchCurrentPath = ''.
    DO minIndex = 1 TO NUM-ENTRIES(cchModulePaths) BY 2:
      IF ENTRY(minIndex,cchModulePaths) EQ ipchNombre THEN DO:
        cchCurrentPath = ENTRY(minIndex + 1,cchModulePaths).
        LEAVE.
      END.
    END.
  END METHOD.

  /* devuelve la ruta completa al archivo, segn el mapeo de mdulos. Si el archivo
  no es un .p devuelve una cadena vaca */
  METHOD PROTECTED CHARACTER Normalizar(INPUT ipchNombre AS CHARACTER):
    DEFINE BUFFER mbfXref FOR xrefsrc.xrfprg.

    IF NUM-ENTRIES(ipchNombre,'.') LT 2 OR ENTRY(2,ipchNombre,'.') NE 'p' THEN
      RETURN ''.
    FOR FIRST mbfXref WHERE xrffil MATCHES '*' + ipchNombre:
      ipchNombre = xrffil.
    END.
    IF SUBSTRING(ipchNombre,1,1) EQ '.' THEN
      ENTRY(1,ipchNombre,'/') = ''.
    IF SUBSTRING(ipchNombre,1,1) EQ '/' THEN
      ipchNombre = SUBSTRING(ipchNombre,2).
    RETURN cchCurrentPath + ipchNombre.
  END METHOD.

  /* quita la informacin utilizada para el armado de menes */
  METHOD PROTECTED CHARACTER Limpiar(INPUT ipchDesc AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    IF SUBSTRING(ipchDesc,2,1) EQ '.' THEN
      ipchDesc = SUBSTRING(ipchDesc,3).
    ELSE DO:
      minIndex = INDEX(ipchDesc,'-').
      IF minIndex GT 0 THEN DO:
        minIndex = INTEGER(SUBSTRING(ipchDesc,1,minIndex)) NO-ERROR.
        IF NOT ERROR-STATUS:ERROR THEN
          ipchDesc = SUBSTRING(ipchDesc,INDEX(ipchDesc,'-') + 1).
      END.
    END.
    RETURN TRIM(ipchDesc).
  END METHOD.

  METHOD PROTECTED VOID DoModule(INPUT ipchModulo AS CHARACTER):
    DEFINE BUFFER mbfMS FOR mapsissrc.mapsis. 
    DEFINE VARIABLE mchContent AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchpath AS CHARACTER   NO-UNDO.
    
    FOR EACH mbfMS WHERE nomsis EQ ipchModulo BREAK BY nomprg ON ERROR UNDO, THROW:
      IF FIRST-OF(nomprg) THEN
        mchContent = ''.
      mchContent = mchContent + ' | ' + Limpiar(desprg).
      IF LAST-OF(nomprg) THEN DO:
        SUBSTRING(mchContent,1,3) = ''.
        mchpath = Normalizar(nomprg).
        IF mchpath EQ '' THEN
          NEXT.
        OutputTag('alias','path,' + mchpath,mchContent).
      END.
    END.
  END METHOD.

  METHOD PUBLIC OVERRIDE VOID writeXML(INPUT ipchDestino AS CHARACTER):
    DEFINE BUFFER mbfModulo FOR mapsissrc.mapsis.
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
  
    StartXML('alias','generador,NomadeSoft XRef',ipchDestino).
    FOR EACH mbfModulo BREAK BY nomsis ON ERROR UNDO, THROW:
      IF FIRST-OF(nomsis) THEN DO:
        StartTag('modulo','name,' + nomsis).
        SetModulePath(nomsis).
        DoModule(nomsis).
        endTag().
      END.
    END.

    FINALLY:
      FinishXML().
    END FINALLY.
  END METHOD.
END CLASS.
